/*
 * alarms.c
 *
 * This tool reads and sets the alarms panel on an Intel Telco chassis.
 * Note that the Intel Server Management software will set these alarms
 * based on firmware-detected thresholds and events.
 * 
 * Author:  Andy Cress  andrew.r.cress@intel.com
 * Copyright (c) 2003 Intel Corporation. 
 *
 * 02/25/03 Andy Cress - created
 * 04/08/03 Andy Cress - added -i for ChassisIdentify
 * 04/30/03 Andy Cress - only try to set ID on/off if option specified
 */
/*M*
Copyright (c) 2003, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

  a.. Redistributions of source code must retain the above copyright notice, 
      this list of conditions and the following disclaimer. 
  b.. Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation 
      and/or other materials provided with the distribution. 
  c.. Neither the name of Intel Corporation nor the names of its contributors 
      may be used to endorse or promote products derived from this software 
      without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *M*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#include "ipmicmd.h"
 
/*
 * Global variables 
 */
char   fdebug    = 0;
char * progver   = "1.1";
char * progname  = "alarms";

static uchar get_alarms(void)
{
	uchar inputData[4];
	uchar responseData[16];
	int responseLength = 4;
	uchar completionCode;
	int ret;

	inputData[0] = 0x03;  // PRIVATE_BUS_ID
	inputData[1] = 0x41;  // ALARMS_PANEL_READ
	inputData[2] = 0x1;   // return one byte of alarms data
	inputData[3] = 0x00;  // init data to zero
        ret = ipmi_cmd(MASTER_WRITE_READ, inputData, 3, responseData,
                        &responseLength, &completionCode, fdebug);
	if (ret != 0) {
           printf("get_alarms: ret = %d, ccode %02x, alarms = %02x\n",
                    ret, completionCode, responseData[0]);
	   return(0);
	   }
	return(responseData[0]);
}  /*end get_alarms()*/

static int chassis_id(uchar val)
{
	uchar inputData[4];
	uchar responseData[16];
	int responseLength = 4;
	uchar completionCode;
	int ret;

	inputData[0] = val;   /* #seconds to turn on id, 0=turn off */
        ret = ipmi_cmd(CHASSIS_IDENTIFY, inputData, 1, responseData,
                        &responseLength, &completionCode, fdebug);
	if (ret != 0) {
           printf("chassis_id: ret = %d, ccode %02x, value = %02x\n",
                    ret, completionCode, val);
	   return(ret);
	   }
	if (completionCode != 0) ret = completionCode;
	return(ret);
} /*end chassis_id*/

static int set_alarms(uchar val)
{
	uchar inputData[4];
	uchar responseData[16];
	int responseLength = 4;
	uchar completionCode;
	int ret;

	inputData[0] = 0x03;  // PRIVATE_BUS_ID
	inputData[1] = 0x40;  // ALARMS_PANEL_WRITE
	inputData[2] = 0x1;   // one byte of alarms data
	inputData[3] = val;  
        ret = ipmi_cmd(MASTER_WRITE_READ, inputData, 4, responseData,
                        &responseLength, &completionCode, fdebug);
	if (ret != 0) {
           printf("set_alarms: ret = %d, ccode %02x, value = %02x\n",
                    ret, completionCode, val);
	   return(ret);
	   }
	if (completionCode != 0) ret = completionCode;
	return(ret);
}  /*end set_alarms()*/

void show_alarms(uchar val)
{
   char *scrit = "ON ";
   char *smaj = "ON ";
   char *smin = "ON ";
   char *spow = "ON ";
   char *rmaj = "off";
   char *rmin = "off";
   if (fdebug) printf("alarms = %02x\n",val);
    
   if (val & 0x01) spow = "off";
   if (val & 0x02) scrit = "off";
   if (val & 0x04) smaj = "off";
   if (val & 0x08) smin = "off";
   printf("alarms: critical = %s major = %s minor = %s power = %s\n",
           scrit,smaj,smin,spow);
   if ((val & 0xc0) == 0) 
	printf("relays: disabled\n");
   else {
	if ((val & 0x10) == 0) rmaj = "ON ";
	if ((val & 0x20) == 0) rmin = "ON ";
	printf("relays: enabled, major = %s minor = %s\n", rmaj, rmin);
   }
}

int
main(int argc, char **argv)
{
   int ret = 0;
   char c;
   uchar fsetled = 0;
   uchar fcrit = 0;
   uchar fmaj = 0;
   uchar fmin = 0;
   uchar fpow = 0;
   uchar fsetid = 0;
   uchar fid = 0;
   uchar newvalue = 0xff;
   uchar alarms = 0;

   printf("%s ver %s\n", progname,progver);

   while ( (c = getopt( argc, argv,"rxc:m:n:p:i:o?")) != EOF ) 
      switch(c) {
          case 'r': fsetled=0; fsetid=0; break; /* read only */
          case 'c': fcrit = atoi(optarg);     /* set critical alarm value */
		    fsetled = 1; break; 
          case 'm': fmaj = atoi(optarg);      /* set major alarm value */
		    fsetled = 1; break; 
          case 'n': fmin = atoi(optarg);      /* set minor alarm value */
		    fsetled = 1; break; 
          case 'p': fpow = atoi(optarg);      /* set power alarm value */
		    fsetled = 1; break; 
          case 'i': fid = atoi(optarg);       /* set chassis id on/off */
		    fsetid=1; break; 
          case 'o': fcrit=0; fmaj=0; fmin=0; fpow=0; /* set all alarms off */
		    fsetled = 1; fsetid=1; fid=0; break; 
          case 'x': fdebug = 1;     break;  /* debug messages */
	  default:
                printf("Usage: %s [-r -i -c -m -n -p -o -x]\n", progname);
                printf(" where -r   means read-only\n");
                printf("       -i5  sets Chassis ID on for 5 sec\n");
                printf("       -i0  sets Chassis ID off\n");
                printf("       -c1  sets Critical Alarm on\n");
                printf("       -c0  sets Critical Alarm off\n");
                printf("       -m1  sets Major Alarm on\n");
                printf("       -m0  sets Major Alarm off\n");
                printf("       -n1  sets Minor Alarm on\n");
                printf("       -n0  sets Minor Alarm off\n");
                printf("       -p1  sets Power Alarm on\n");
                printf("       -p0  sets Power Alarm off\n");
                printf("       -o   sets all Alarms off\n");
                printf("       -x   show eXtra debug messages\n");
                exit(1);
      }

   alarms = get_alarms();
   if (alarms == 0) {
	printf("Could not obtain alarm states, may not be present.\n\n");
	exit(-1);
	}
   else {
	show_alarms(alarms);
   }
   if (fsetid) {
	printf("Setting id to %d ...\n\n",fid);
	ret = chassis_id(fid);
   }
   if (fsetled) {
      if (fcrit) newvalue = newvalue & 0xFD; /*bit1 = 0*/
      if (fmaj)  newvalue = newvalue & 0xFB; /*bit2 = 0*/
      if (fmin)  newvalue = newvalue & 0xF7; /*bit3 = 0*/
      if (fpow)  newvalue = newvalue & 0xFE; /*bit0 = 0*/
      printf("\nSetting alarms to %02x ...\n",newvalue);
      ret = set_alarms(newvalue);
      alarms = get_alarms();
      show_alarms(alarms);
   }
   printf("\n");
   exit (ret);
}  /* end main()*/

/* end alarms.c */
